@@ -1,7 +1,7 @@
 // ------------------------------------------------------------------------
 //
 // SPDX-License-Identifier: LGPL-2.1-or-later
-// Copyright (C) 1998 - 2020 by the deal.II authors
+// Copyright (C) 1998 - 2025 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -15,9 +15,12 @@
 
 #include <deal.II/numerics/vector_tools_interpolate.templates.h>
 
+#include <map>
+
+
 DEAL_II_NAMESPACE_OPEN
 
 // ---------------------------- explicit instantiations --------------------
-#include "vector_tools_interpolate.inst"
+#include "numerics/vector_tools_interpolate.inst"
 
 DEAL_II_NAMESPACE_CLOSE
